namespace eval  {}

set dataDir [file normalize [file join [pwd] etc]]
set dataFile [file join $dataDir LicenseManager.par]

set portNumber 8090
set totalConnectionCount 0
##
## Put trace on license count to invoke alarm checking
##
trace variable totalConnectionCount w CheckAlarms

array set connectionByExecutableArray {}
array set socketDataArray {}
array set hostDataArray {}


###########################################################################
#
# Procedure Name : AcceptConnection
#
# Description : This accepts a connection and sets up the read event.
#
# Arguments :
#     socket -- The channel name for the socket.
#     host   -- The originating host, most likely as an IP address
#     port   -- The originating port, most likely as a number
#
# Returns : nonthing
#
# Side-Effects : none
#
# Exception Conditions : none
#
# Pre-requisite Conditions :
#
# Original Author : Gerald W. Lester
#
###########################################################################
proc AcceptConnection {socket host port} {
    variable socketDataArray

    fconfigure $socket -translation binary -blocking 0
    set socketDataArray($socket,state) header
    set socketDataArray($socket,request) {}
    set socketDataArray($socket,bytesLeft) 10
    set socketDataArray($socket,executable) {}
    set socketDataArray($socket,host) $host
    fileevent $socket readable [list GetIncomingRequest $socket]
    return
}


###########################################################################
#
# Procedure Name : GetIncomingRequest
#
# Description : This reads data and when an entire message is received invokes
#               the procedure to deal with the message.
#
#               Legal messages are:
#                       HELLO <executable>
#                       DISCONNECT
#
# Arguments :
#     socket -- The channel name for the socket.
#
# Returns : nonthing
#
# Side-Effects : none
#
# Exception Conditions : none
#
# Pre-requisite Conditions :
#
# Original Author : Gerald W. Lester
#
###########################################################################
proc GetIncomingRequest {socket} {
    variable socketDataArray

    if {[llength [file channel $socket]] && ![eof $socket]} {
        ##
        ## We have a valid open channel, so process the request.
        ##
        ## Ignore any I/O error as they will get processed at the end.
        ##
        catch {
            set str [read $socket $socketDataArray($socket,bytesLeft)]
            append socketDataArray($socket,request) $str


            incr socketDataArray($socket,bytesLeft) -[string length $str]
            if {$socketDataArray($socket,bytesLeft) == 0} {
                switch $socketDataArray($socket,state) {
                    header {
                        scan $socketDataArray($socket,request) {%d} socketDataArray($socket,bytesLeft)
                        set socketDataArray($socket,state) body
                    }
                    body {
                        set reply {}
                        set command [lindex $socketDataArray($socket,request) 0]
                        set data [lindex $socketDataArray($socket,request) 1]
                        switch -exact $command {
                            HELLO {
                                ## puts stdout [list Connect $socket $data]
                                Connect $socket $data
                            }
                            DISCONNECT {
                                ## puts stdout [list Disconnect $socket]
                                Disconnect $socket
                            }
                        }

                        set socketDataArray($socket,state) header
                        set socketDataArray($socket,bytesLeft) 10
                    }
                }
                set socketDataArray($socket,request) {}
            }
        }
    }
    if {[llength [file channel $socket]] && [eof $socket]} {
        Disconnect $socket
    }
    return
}


###########################################################################
#
# Procedure Name : Connect
#
# Description : This handles when a new connections is confirmed.
#
# Arguments :
#     socket -- The channel name for the socket.
#     prgm   -- The name of the program.
#
# Returns : nonthing
#
# Side-Effects : none
#
# Exception Conditions : none
#
# Pre-requisite Conditions :
#
# Original Author : Gerald W. Lester
#
###########################################################################
proc Connect {socket prgm} {
    variable socketDataArray
    variable connectionByExecutableArray
    variable totalConnectionCount
    variable hostDataArray

    set socketDataArray($socket,executable) $prgm
    if {[string length $prgm]} {
        if {![info exists connectionByExecutableArray($prgm)]} {
            set connectionByExecutableArray($prgm) 0
        }
        set host $socketDataArray($socket,host)
        set hostDataArray($host,socket,$socket) $socket
        incr connectionByExecutableArray($prgm) 1
        if {[llength [array names hocstDataArray $host,socket,*]] != 2} {
            incr totalConnectionCount 1
        }
    }
    return
}


###########################################################################
#
# Procedure Name : Disconnect
#
# Description : Handles when a client exits.
#
# Arguments :
#     socket -- The channel name for the socket.
#
# Returns : nonthing
#
# Side-Effects : none
#
# Exception Conditions : none
#
# Pre-requisite Conditions :
#
# Original Author : Gerald W. Lester
#
###########################################################################
proc Disconnect {socket} {
    variable socketDataArray
    variable connectionByExecutableArray
    variable totalConnectionCount
    variable hostDataArray

    catch {fileevent $socket readable {}}
    catch {close $socket}
    set prgm $socketDataArray($socket,executable)
    if {[string length $prgm] && [info exists connectionByExecutableArray($prgm)]} {
        incr connectionByExecutableArray($prgm) -1
    }
    set host $socketDataArray($socket,host)
    if {[llength [array names hocstDataArray $host,socket,*]] != 2} {
        incr totalConnectionCount -1
    }
    unset hostDataArray($host,socket,$socket)
    array unset socketDataArray $socket,*
    return
}


###########################################################################
#
# Procedure Name : WriteAlarmSettings
#
# Description : Writes the alarm settigns for later use..
#
# Arguments : none
#
# Returns : nonthing
#
# Side-Effects : none
#
# Exception Conditions : none
#
# Pre-requisite Conditions :
#
# Original Author : Gerald W. Lester
#
###########################################################################
proc SwitchPorts {} {
    variable listeningSocket
    variable portNumber

    catch {close $listeningSocket}

    set listeningSocket \
        [socket -server AcceptConnection $portNumber]
}


###########################################################################
#
# Procedure Name : WriteAlarmSettings
#
# Description : Writes the alarm settigns for later use..
#
# Arguments : none
#
# Returns : nonthing
#
# Side-Effects : none
#
# Exception Conditions : none
#
# Pre-requisite Conditions :
#
# Original Author : Gerald W. Lester
#
###########################################################################
proc WriteConfiguration {} {
    WriteLog  {Saving configuration parameters}
    set ofd [open $dataFile {WRONLY TRUNC CREAT}]
    puts $ofd [list set portNumber $portNumber]
    puts $ofd [list set refreshRate $refreshRate]
    puts $ofd [list set logFileName $logFileName]
    puts $ofd [list WriteLog {Starting License Manager}]
    puts $ofd [list SwitchPorts]
    close $ofd
}


##
## Load existing configuration settings
##
if {[file exists $dataFile]} {
    catch {source $dataFile}
} else {
    ##
    ## Write the default ones out
    ##
    WriteConfiguration
}
##
## Load existing alarm settings
##
if {[file exists $dataFile]} {
    WriteLog "Reading alarm definitions from $dataFile"
    catch {source $dataFile}
} else {
    ##
    ## Write the default ones out
    ##
    WriteLog "Writeing default alarm definitions to $dataFile"
    WriteAlarmSettings
}

